<?php

namespace Storage;

class Cookie {

    protected array $options = [
        'name'     => '',
        'expires'  => 0,
        'path'     => null,
        'domain'   => null,
        'secure'   => false,
        'httponly' => true,
        'samesite' => "Strict",
    ];

    protected array $options;

    protected string $secureHash;

    public function __construct(array $options, string $secureHash) {
        $this->options = $options;
        $this->secureHash = $secureHash;
    }

    /**
     * Sets a cookie with the given parameters.
     * @param   string   cookie name or array of config options
     * @param   string   cookie value
     * @param   integer  number of seconds before the cookie expires
     * @param   string   URL path to allow
     * @param   string   URL domain to allow
     * @param   boolean  HTTPS only
     * @param   boolean  HTTP only (requires PHP 5.2 or higher)
     * @return  boolean
     */
    public static function set(
        ?string $name,
        ?string $value = null,
        ?int $expires = null,
        ?string $path = null,
        ?string $domain = null,
        ?bool $secure = null,
        ?bool $httponly = null,
        ?string $samesite = null,
        ?bool $raw = false
    ) {
        if ( headers_sent() ) {
            return false;
        }

        $expire = ($expire == 0) ? 0 : time() + (int) $expire;

        $_COOKIE[$name] = $value;

        $options = [
            'expires'  => $this->options['expires']  ?? ( $expires ?: 0 ),
            'path'     => $this->options['path']     ?? ( $path ?: "" ),
            'domain'   => $this->options['domain']   ?? ( $domain ?: "" ),
            'secure'   => $this->options['secure']   ?? ( $secure ?: false ),
            'httponly' => $this->options['httponly'] ?? ( $httponly ?: false ),
            'samesite' => $this->options['samesite'] ?? ( $samesite ?: "" ),
        ];

        return $raw ? setrawcookie($name, $value ?: "", $options) : setcookie($name, $value ?: "", $options);
    }


    /**
     * Fetch a cookie value, using the Input library.
     * @param   string   cookie name
     * @param   mixed    default value
     * @return  string
     */
    public static function get(string $key, $default = null)
    {
        return array_key_exists($key, $_COOKIE) ? $_COOKIE[$key] : $default;
    }

    /**
     * Nullify and unset a cookie.
     * @param   string   cookie name
     * @param   string   URL path
     * @param   string   URL domain
     * @return  boolean
     */
    public static function delete(string $name, ?string $path = null, ?string $domain = null) {
        if ( ! $this->__isset($name) ) {
            return false;
        }

        unset( $_COOKIE[$name] );

        return static::set($name, '', -86400, $path ?: ( $this->options['path'] ?? "" ), $domain ?: ( $this->options['domain'] ?? "" ), $this->options['secure'] ?? false, $this->options['httponly'] ?? false);
    }

    public function __set($key, $value)
    {
        return static::set($key, $value);
    }

    public function __get($key)
    {
        return static::get($key);
    }

    public function __isset($key)
    {
        return array_key_exists($key, $_SESSION);
    }

    public function __unset($key)
    {
        $this->delete($key);
    }
}
